from ZODB.POSException import ConflictError

class KeyConflictHandler:
    '''Handler class used to handle key value conflicts.'''
    
    def __init__(self, defaultHandlerMethod=None):
        '''Handler initialiser
        defaultHandlerMethod : method that will be used if no specific handler is found for a key.
        Note: defaultHandlerMethod takes two parameters, value1 and value2'''
        self.defaultHandlerMethod = defaultHandlerMethod
        self.handleableKeys = {}
        
    def addHandleableKey(self, keyName, method):
        '''Add handler method for a specific key entry.
        keyName : the key name that can be handled
        method : the method that can handle the key mimatch.
        Note: medhod takes two parameters, value1 and value2'''
        self.handleableKeys[keyName] = method
        
    def canHandleKey(self, keyName):
        '''Returns true if a specific handler exists for keyName'''
        return self.handleableKeys.has_key(keyName)
    
    def handleConflict(self, keyName, value1, value2):
        '''Handle a conflict between two values for a key
        keyName : the key name where the conflict occurs
        value1 : the first value
        value2 : the second value
        Returns the value calculated by the handler as the correct one.'''
        if not self.canHandleKey(keyName):
            if self.defaultHandlerMethod is None:
                raise KeyError('Cannot handle key %s' %keyName)
            else:
                return self.defaultHandlerMethod(value1, value2)
        else:
            return self.handleableKeys[keyName](value1, value2)
        
def mustHaveValue(value1, value2):
    '''Returns the one that has a value.
    If both have a value or both does not have a value raises a ValueError'''
    if value1 is None and value2 is None:
        raise ValueError('Both values have None value!')
    if value1 is not None and value2 is not None:
        if value1 != value2:
            raise ValueError("The values should be the same.\n %s != %s" %(value1, value2))
    if value1 is None:
        return value2
    
    if value2 is None:
        return value1
        
        
def errorIfDifferent(value1, value2):
    '''Raises an error if value1 and value2 are not the same.'''
    if value1 != value2:
        raise ValueError('The values should be the same.\n %s != %s' %(value1, value2))
        
def returnLargest(value1, value2):
    '''Handler method that returns the largest of the two values'''
    if value1 > value2:
        return value1
    else:
        return value2
    
def returnSmallest(value1, value2):
    '''Handler method that returns the smallest of the two values'''
    if value1 < value2:
        return value1
    else:
        return value2
    
def listMerge(value1, value2):
    '''Merges the two values into a list of unique values containing both value1 and
    value2's values.'''    
    retList = []
    for item in value1:
        retList.append(item)
    for item in value2:
        if item not in retList:
            retList.append(item)
    return retList

def tupleMerge(value1, value2):
    '''Merges the two values into a tuple of unique values containing both value1 and
    value2's values.'''    
    return tuple(listMerge(value1, value2))
    
def dictMerge(value1, value2):
    '''Merges the two dictionaries into a dictionary of unique values containing both value1 and
    value2's values.
    This will ensure that the return dict contains all the keys in value1 and value2.  If a key exists
    in both value1 and value2 but with different values, a KeyError will be raised.'''    
    return mergeDiffs({}, value1, value2, KeyConflictHandler())

def synchroniseObjects(retState):
    '''Synchronises the _objects key's values with the keys in the state.
    retState : a Zope state object
    This method ensures that no objects appear in the _objects key that does not exist
    as key=value pairs in the state object.'''
    if retState.has_key('_objects'):
        current_objects = retState['_objects']
        new_objects = []
        for dict in current_objects:
            if retState.has_key(dict['id']):
                new_objects.append(dict)
        retState['_objects'] = tuple(new_objects)

def diffDictionary(dict1, dict2):
    '''Creates a diff between dict1 and dict2
    Returns a tuple (additionDict, removalList, changeDict)'''
    #create a dictionary of values that were added
    additionDict = {}
    for key in dict2.keys():
        if not dict1.has_key(key):
            additionDict[key] = dict2[key]
            
    #create a list of keys that were removed
    removalList = []
    changeDict = {}
    for key in dict1.keys():
        if not dict2.has_key(key):
            removalList.append(key)
        elif dict1[key] != dict2[key]:
            #use dict2's value
            changeDict[key] = dict2[key]
            
    return (additionDict, removalList, changeDict)

def mergeDiffs(originalOb, newOb1, newOb2, handler):
    '''Takes the diff between an original object and two new objects and merges those
    differences into a new object.
    originalOb : a Zope state object reflecting the old state
    newOb1 : a Zope state object reflecting a new state (normally saved state)
    newOb2 : a Zope state object reflecting a new state (normally new sate)
    handler : a KeyConflictHandler object set up to handle expected conflicts in the above states'
        values
    Returns a merged state object with additions added, removals removed and conflicts resolved.
    Throws KeyError if a conflict cannot be resolved because of one of the following reasons:
        1. no handler method specified in handler for a specific key
        2. removal in one state object and value change in the other cannot be resolved.
        '''
    changeDict = {}
    addDict = {}
    removalList = []

    diff1 = diffDictionary(originalOb, newOb1)
    diff2 = diffDictionary(originalOb, newOb2)

    add1,rem1,change1 = diff1
    add2,rem2,change2 = diff2
    
    #changes
    for key in change1.keys():
        if change2.has_key(key):
            if change2[key] == change1[key]:
                changeDict[key] = change1[key]
            else:
                changeDict[key] = handler.handleConflict(key, change1[key], change2[key])
        else:
            changeDict[key] = handler.handleConflict(key, change1[key], originalOb[key])
    for key in change2.keys():
        if not change1.has_key(key):
            changeDict[key] = handler.handleConflict(key, change2[key], originalOb[key])

    #additions
    for key in add1.keys():
        if add2.has_key(key):
            if add2[key] != add1[key]:
                addDict[key] = handler.handleConflict(key, add1[key], add2[key])
            else:
                addDict[key] = add1[key]
        else:
            addDict[key] = add1[key]
            
    for key in add2.keys():
        if not add1.has_key(key):
            addDict[key] = add2[key]
                
    
    #removals
    for removal in rem1:
        if changeDict.has_key(removal):
            raise KeyError('Key %s has been changed AND removed.' %removal)
        else:
            removalList.append(removal)
            
    for removal in rem2:
        if changeDict.has_key(removal):
            raise KeyError('Key %s has been changed AND removed.' %removal)
        elif removal not in removalList:
            removalList.append(removal)
            
    #now apply the diffs
    mergedOb = {}
    
    #add additions
    for key in addDict.keys():
        mergedOb[key] = addDict[key]
        
    #add changes
    for key in changeDict.keys():
        mergedOb[key] = changeDict[key]

    #add non-removed, not-changed keys from the original
    for key in originalOb.keys():
        if key not in removalList and not changeDict.has_key(key):
            mergedOb[key] = originalOb[key]
            
    return mergedOb


if __name__ == "__main__":
    
    o = {'_objects':('ob1','ob2'),
         'a':1,'b':2,'c':3}
    s = {'_objects':('ob1',),
        'a':1,'c':3,'d':4}
    n = {'_objects':('ob1','ob3'),
        'c':4,'d':4,'e':5}
        
    handler = KeyConflictHandler(returnLargest)
    handler.addHandleableKey('_objects', tupleMerge)
    handler.addHandleableKey('c', returnLargest)
    
    newDict = mergeDiffs(o, s, n, handler)
    
    print newDict
            
    

