import unittest
import pyMQ
import xmlrpclib
import time

counter = 0
port = 7123

class pyMQTest(unittest.TestCase):
    def setUp(self):
        self.s1, self.ps, self.nodeNameS, self.portS = self.getServerAndConnector(1)
        self.s2, self.rs, self.nodeNameR, self.portR = self.getServerAndConnector(2)
        self.s1.addRemoteNode('pyMQ1','http://127.0.0.1:%s' %self.portS)
        self.s2.addRemoteNode('pyMQ0','http://127.0.0.1:%s' %self.portR)
        self.s1.start()
        self.s2.start()
    
    def getServerAndConnector(self, number):
        global counter
        global port
        nodeName = 'pyMQNode%s%s' %(counter, number)
        pyMQNode = pyMQ.MQServer()
        pyMQNode.load('TestData', nodeName, '127.0.0.1', port)    
        xServer = xmlrpclib.Server('http://127.0.0.1:%s' %port)
        counter += 1
        port += 1
        return (pyMQNode, xServer, nodeName, port)
        

    def tearDown(self):
        self.s1.stop()
        self.s2.stop()
        del self.s1
        del self.s2
##        os.remove(os.path.join('.',self.nodeNameS))
##        os.remove(os.path.join('.',self.nodeNameR))

    def testPublishSingle(self):
        '''Publishing of one message.'''
        result = self.ps.publishMessagesToTopic('topic','test',['Message1'])
        self.assertEqual(result,[0])

    def testPublishMultiple(self):
        '''Publishing of 100 messages.'''
        ml = ['Message%s' %i for i in range(100)]
        result = self.ps.publishMessagesToTopic('topic','test',ml)
        self.assertEqual(result,range(100))
        
    def testSubscription(self):
        '''Test subscribing to a Topic'''
        result = self.rs.subscribedToTopic('topic','test')
        self.assertEqual(result, 0, 'Must not be initially subscribed.')

        result = self.rs.subscribeToTopic('topic','test')
        self.assertEqual(result, 1)

        result = self.rs.subscribedToTopic('topic','test')
        self.assertEqual(result, 1, 'Must now be subscribed.')

        result = self.rs.subscribeToTopic('topic','test')
        self.assertEqual(result, 0, 'Subscribing while being subscribed must return 0')

        result = self.rs.unsubscribeFromTopic('topic','test')
        self.assertEqual(result, 1, 'Must now be unsubscribed.')
        
        result = self.rs.subscribedToTopic('topic','test')
        self.assertEqual(result, 0, 'Must now be subscribed.')
        
        result = self.rs.unsubscribeFromTopic('topic','test')
        self.assertEqual(result, 0, 'Unsubscribing while unsubscribed must return a zero.')
        
    def testAutoTopicAndSubscription(self):
        '''Creates topics and subscriptions automatically if they don't exist'''
        self.assertEqual(self.ps.publishMessagesToTopic('b','test',['1','2']), [0,1])
        self.assertEqual(self.ps.getMessagesFromTopic('c','test'),[])
        
        self.failIf(self.ps.subscribedToTopic('b','test'))
        self.failUnless(self.ps.subscribedToTopic('c','test'))
    
    def testMessageTransfer(self):
        '''Tests message publish/receive'''
        sMessages = ['Message%s' %i for i in range(100)]
        sMessages.sort()
        self.ps.publishMessagesToTopic('topic','test',sMessages)
        
        messages = []
        t = time.time()
        while len(messages) < 100 and time.time()-t < 10:
            rMessages = self.rs.getMessagesFromTopic('topic','test')
            messages += [message for id, message in rMessages]
            self.rs.acknowledgeMessagesFromTopic('topic','test',[id for id, message in rMessages])
            time.sleep(1)
        messages.sort()
        o = ""
        for i in range(100):
            if len(messages) > i:
                o += "%s : %s\n" %(sMessages[i], messages[i])
            else:
                o += "%s\n" %sMessages[i]
        self.failUnless(sMessages==messages, o)

    def testServerStartAndCleanup(self):
        '''Server start/stop/start/stop cleanly'''
        global counter
        global port
        nodeName = 'pyMQNode%s%s' %(counter, 3)
        pyMQNode = pyMQ.MQServer()
        pyMQNode.load('TestData', nodeName, '127.0.0.1', port)    
        xServer = xmlrpclib.Server('http://127.0.0.1:%s' %port)
        counter += 1
        port += 1
        
        pyMQNode.start()
        time.sleep(2)
        pyMQNode.stop()
        pyMQNode.unload()
        time.sleep(2)
        del pyMQNode
        
        pyMQNode = pyMQ.MQServer()
        pyMQNode.load('TestData', nodeName, '127.0.0.1', port)    
        pyMQNode.start()
        time.sleep(2)
        pyMQNode.stop()
        pyMQNode.unload()
        time.sleep(2)
        del pyMQNode
        
    def testPublishStopStartDelivery(self):
        '''Server delivers after stop/start cycle'''
        global counter
        global port
        nodeName = 'pyMQNode%s%s' %(counter, 3)
        pyMQNode = pyMQ.MQServer()
        pyMQNode.load('TestData', nodeName, '127.0.0.1', port)    
        pyMQNode.start()
        xServer = xmlrpclib.Server('http://127.0.0.1:%s' %port)
        counter += 1
        port += 1
        time.sleep(2)

        # Make two subscribers
        xServer.subscribeToTopic("topic","testReceive")
        xServer.subscribeToTopic("topic","testReceive2")
        # Publish some messages
        xServer.publishMessagesToTopic("topic","test",["MESSAGE"])
        # Check that subscriber two receives messages directly and ack messages
        time.sleep(2)
        self.assertEqual(xServer.getMessagesFromTopic("topic","testReceive2"),
            [[0, 'MESSAGE']])            
        xServer.acknowledgeMessagesFromTopic("topic","testReceive2",[0])
        # Check that we don't receive any messages after ack again
        self.assertEqual(xServer.getMessagesFromTopic("topic","testReceive2"),
            [])

        pyMQNode.stop()
        pyMQNode.unload()
        time.sleep(2)
        del pyMQNode
        port += 100
        time.sleep(2)
        pyMQNode = pyMQ.MQServer()
        pyMQNode.load('TestData', nodeName, '127.0.0.1', port)    
        pyMQNode.start()
        time.sleep(2)
        
        # Check that we are still subscribed after shutdown/restart
        self.failUnless(xServer.subscribedToTopic("topic","testReceive"),
            "Should still be subscribed after shutdown")
        # Check that we get messages after shutdown/restart and ack messages
        self.assertEqual(xServer.getMessagesFromTopic("topic","testReceive"),
            [[0, 'MESSAGE']], 'Did not receive the message')
        self.acknowledgeMessagesFromTopic("topic","testReceive",[0])
        # Check that we don't receive messages after ack
        self.assertEqual(xServer.getMessagesFromTopic("topic","testReceive"),
            [])
        # Check that we don't receive messages after ack and shutdown/restart
        self.assertEqual(xServer.getMessagesFromTopic("topic","testReceive2"),
            [])
        
        pyMQNode.stop()
        time.sleep(2)
        del pyMQNode        
        
        
        

if __name__ == "__main__":
    import sys, os
    if not os.path.exists("TestData"):
        os.makedirs("TestData")
    else:
        files = os.listdir("TestData")
        for fn in files:
            try:
                os.remove(os.path.join("TestData",fn))
            except:
                pass
        
    sys.argv.append("-v")
    unittest.main()

